"""
Import the ALMA OUS structure in bulk and assign them to EBs that exist
in the archive schema currently.
"""
from io import StringIO

"""
Commentary from Jim:
---

The table we found which looked like the easiest source was: 
ALMA.ASA_OUS. (pick out the MOUSes by ASA_OUS_ID = MEMBER_OUS_ID) and 
you can get all three layers.

The SB name for that table should come from BMMV_SCHEDBLOCK.
"""

import pyat.schema

# Global database connections
ALMA = pyat.schema.create_session('ALMA').connection().engine.raw_connection()
SDM = pyat.schema.create_session('SDM').connection().engine.raw_connection()


def main():
    # Step 1: Read the OUS structure and SB names into a temp table
    obtain_ous_structure()

    # Step 2: Repopulate the OUS table
    repopulate_ous_table()

    # Step 3: Link up the EBs to their MOUSes
    map_ebs_to_mouses()


def obtain_ous_structure():
    """Generates a temporary table named ous_importer_temp which contains the relationships we need and the SB name"""

    # this query obtains the raw data we need; the COALESCE statements here give us a string that is amenable to being
    # interpreted as a null by the COPY Postgres command down below
    ous_query = r"""select 
      asa_project_code, COALESCE(sg_ous_uid, '\N'), COALESCE(group_ous_uid, '\N'), member_ous_uid, sb_name 
    from ASA_OUS ous
    join BMMV_SCHEDBLOCK sb on ous.member_ous_uid = sb.mous_status_uid
    where ous.asa_ous_id = ous.member_ous_uid and sb.status NOT IN ('Suspended', 'Cancelled')"""
    cur = ALMA.cursor()

    # this is a little trick for building something you can pass to cur.copy_from
    copybuffer = StringIO()
    print('Getting OUS structure from ALMA...')
    for row in cur.execute(ous_query):
        copybuffer.write('\t'.join(row) + '\n')
    copybuffer.seek(0)

    cur = SDM.cursor()
    print('Creating ous_importer_temp')
    cur.execute('drop table if exists ous_importer_temp')
    cur.execute(
        """create temporary table ous_importer_temp (
    asa_project_code varchar, 
    sg_ous_uid varchar, 
    group_ous_uid varchar, 
    member_ous_uid varchar, 
    sb_name varchar)""")

    print('Copying into ous_importer_temp')
    cur.copy_from(copybuffer, 'ous_importer_temp')
    SDM.commit()


def repopulate_ous_table():
    cur = SDM.cursor()

    delete_missing = """delete
from ous_importer_temp
where asa_project_code in
      (select alma.asa_project_code
       from ous_importer_temp alma
              left join projects p on alma.asa_project_code = p.project_code
       where p.project_code is null)"""

    print('Excluding OUSes with no corresponding project in our schema...')
    cur.execute(delete_missing)

    insert_missing = """insert into alma_ouses
select
       sg_ous_uid as alma_ous_id,
       null as parent_ous_id,
       asa_project_code as project_code,
       'SOUS' as ous_type,
       null as schedblock_name
from ous_importer_temp
where sg_ous_uid is not null
group by sg_ous_uid, asa_project_code
union all
select
       group_ous_uid as alma_ous_id,
       sg_ous_uid as parent_ous_id,
       null as project_code,
       'GOUS' as ous_type,
       null as schedblock_name
from ous_importer_temp
where group_ous_uid is not null
group by alma_ous_id, sg_ous_uid
union all
select
       t.member_ous_uid as alma_ous_id,
       group_ous_uid as parent_ous_id,
       null as project_code,
      'MOUS' as ous_type,
       sb_name as schedblock_name
from ous_importer_temp
join (select member_ous_uid from ous_importer_temp group by member_ous_uid having count(*) = 1) t
  on t.member_ous_uid = ous_importer_temp.member_ous_uid
where t.member_ous_uid is not null
on conflict (alma_ous_id) do update
  set
    alma_ous_id = excluded.alma_ous_id,
    parent_ous_id = excluded.parent_ous_id,
    project_code = excluded.project_code,
    ous_type = excluded.ous_type,
    schedblock_name = excluded.schedblock_name"""

    print('Insert or updating found OUSes into our schema')
    cur.execute(insert_missing)
    SDM.commit()


def map_ebs_to_mouses():
    cur = ALMA.cursor()

    eb_mous_query = """SELECT execblockuid, sbs.mous_status_uid FROM ALMA.AQUA_EXECBLOCK ebs 
JOIN ALMA.SHIFTLOG_ENTRIES shifts ON ebs.EXECBLOCKUID=shifts.SE_EB_UID 
JOIN ALMA.BMMV_SCHEDBLOCK sbs ON shifts.SE_SB_ID=sbs.ARCHIVE_UID 
WHERE ebs.QA0STATUS IN ('Pass', 'SemiPass')"""

    copybuffer = StringIO()
    print('Getting MOUS<->EB mapping from ALMA...')
    for row in cur.execute(eb_mous_query):
        copybuffer.write('\t'.join(row) + '\n')
    copybuffer.seek(0)

    cur = SDM.cursor()
    print('Creating EB->MOUS map table')
    cur.execute('drop table if exists ebs_mous_map_temp')
    cur.execute(
        'create temporary table ebs_mous_map_temp (execblockuid varchar, mous_status_uid varchar)')
    print('Copying into EB->MOUS map table')
    cur.copy_from(copybuffer, 'ebs_mous_map_temp')

    eb_update = """update execution_blocks
set alma_ous_id = map.mous_status_uid
from ebs_mous_map_temp map,
     alma_ouses
where ngas_fileset_id = replace(replace(map.execblockuid, ':', '_'), '/', '_')
  and alma_ouses.alma_ous_id = map.mous_status_uid
  and band_code is not null"""
    print('Updating execution_blocks with MOUSes from the map table')
    cur.execute(eb_update)

    SDM.commit()
