''' Unit tests for data-fetcher. '''
import logging
import os
import tempfile
from pathlib import Path

import pytest
from datafetcher.datafetcher_refactor import DataFetcherRefactor, ReturnCode
from datafetcher.utilities import get_arg_parser, ProductLocatorLookup, \
    ExecutionSite, RetrievalMode, Location, Cluster
from pycapo import CapoConfig

from .testing_utils import get_test_capo_settings, get_metadata_db_settings, \
    TEST_PROFILE, get_mini_locations_file, get_locations_file, LOCATION_REPORTS

_LOCATION_FILENAME = 'locations.json'
_ASDM_XML = 'ASDM.xml'
_EB_EXTERNAL_NAME = 'sysstartS.58955.83384832176'

# set this to False when debugging one or more tests
# so as not to have to sit thru every test;
# comment out the target test(s)' @pytest.skip
RUN_ALL = True

# pylint: disable=C0115, C0116, R0902, R0903, R0914, W1203, W0613, W0621

@pytest.fixture(autouse=True, scope='function')
def make_tempdir() -> None:
    ''' Creates a new temporary working directory for each test. '''
    umask = os.umask(0o000)
    top_level = tempfile.mkdtemp(prefix='datafetcher_test_', dir='/var/tmp')
    os.umask(umask)
    logging.debug(f'>>> tempdir: {top_level}')
    yield top_level

@pytest.fixture(scope='module')
def capo_settings():
    ''' get Capo settings once for whole module '''

    def retrieve_capo_settings() -> CapoConfig:
        return get_test_capo_settings()

    capo_settings = retrieve_capo_settings()
    yield capo_settings

@pytest.fixture(scope='module')
def settings(capo_settings):
    ''' grabs all the settings we will need for the datafetcher:
        Capo, database, test data
    '''
    db_settings = get_metadata_db_settings(TEST_PROFILE)
    test_data = _initialize_test_data(db_settings)
    yield Settings(capo_settings, db_settings, test_data)

def launch_datafetcher(args, capo_settings):
    ''' invoke the DF with these args as in df.main(),
        launch it with df.run(),
        and return the appropriate return/error code

    '''

    try:
        namespace = get_arg_parser().parse_args(args)
    except SystemExit as exc:
        print(f'{exc}')
        if '--location-file' in args and '--product-locator' in args:
            # you can have one but not both
            raise exc
        if '--profile' not in args \
                or '--profile' not in capo_settings.keys():
            return_code = ReturnCode.MISSING_PROFILE.value['code']
            raise SystemExit(return_code)
        raise exc

    return DataFetcherRefactor(namespace, capo_settings).run()

def test_settings_setup(settings):
    ''' Ensure that the test settings we're using make sense '''
    assert settings.capo_settings is not None
    assert isinstance(settings.capo_settings, dict)
    assert settings.db_settings is not None
    assert isinstance(settings.db_settings, dict)
    assert settings.test_data is not None
    assert isinstance(settings.test_data, dict)

def _initialize_test_data(db_settings):
    ''' Set up test data for use in several tests '''
    ext_name = '13B-014.sb28862036.eb29155786.56782.5720116088'

    product_locator = ProductLocatorLookup(db_settings) \
        .look_up_locator_for_ext_name(ext_name)
    return {'external_name': ext_name,
            'product_locator': product_locator}

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_usage_statement_makes_sense():
    ''' Ensure that the datafetcher's "usage" statement is as we expect

    '''

    usage = DataFetcherRefactor._build_usage_message()
    assert usage.startswith('Usage:')
    lines = usage.split('\n')
    assert len(lines) >= len(ReturnCode) + 1

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_no_args_prints_usage_and_croaks():
    ''' If the datafetcher is called without any arguments,
        it should print the "usage" statement, then exit

    '''

    with pytest.raises(TypeError):
        datafetcher = DataFetcherRefactor()
        assert datafetcher.usage == DataFetcherRefactor._build_usage_message()

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_omitted_capo_args_returns_expected_code(make_tempdir, settings):
    ''' Be sure DF dies with appropriate error message if called without Capo profile '''

    args = ['--product-locator', 'not-even',
            '--output-dir', str(make_tempdir)]
    with pytest.raises(SystemExit) as exc:
        launch_datafetcher(args, settings.capo_settings)
    assert exc.value.code['code'] == 1

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_omitted_capo_value_returns_expected_code(make_tempdir, settings):
    ''' Be sure DF dies with appropriate error message if Capo profile
        argument but no profile value

    '''

    args = ['--product-locator', "we're not going to get this far",
            '--output-dir', str(make_tempdir),
            '--profile']
    expected = ReturnCode.MISSING_PROFILE.value['code']
    with pytest.raises(SystemExit) as exc:
        launch_datafetcher(args, settings.capo_settings)
    assert exc.value.code == expected

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_invalid_capo_profile_returns_expected_code(make_tempdir, settings):
    ''' Be sure DF dies with appropriate error message if called without Capo profile '''

    args = ['--profile', 'whatevs',
            '--product-locator', "we'll never get this far",
            '--output-dir', str(make_tempdir),
            ]
    expected = ReturnCode.MISSING_PROFILE.value['code']
    with pytest.raises(SystemExit) as exc:
        launch_datafetcher(args, settings.capo_settings)
    assert exc.value.code['code'] == expected

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_inaccessible_output_dir_returns_expected_code(settings, make_tempdir):
    umask = os.umask(0o000)
    tmpdir = Path(make_tempdir)
    tmpdir.chmod(0o666)
    args = ['--product-locator', 'scooby-doo',
            '--profile', TEST_PROFILE,
            '--output-dir', str(make_tempdir)]
    original_settings = settings.capo_settings
    try:
        with pytest.raises(SystemExit) as exc:
            settings.capo_settings.pop('download_dir')
            launch_datafetcher(args, settings.capo_settings)
        assert exc.value.code == ReturnCode.MISSING_SETTING.value
    finally:
        tmpdir.chmod(umask)
        settings.capo_settings = original_settings
        # this test results in a log file being written to the
        # current directory. nuke it.
        curdir = Path('.')
        log_files = [file for file in curdir.iterdir()
                     if str(file).endswith('.log')]
        for file in log_files:
            file.unlink()

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_two_locator_args_returns_expected_code(make_tempdir, settings):
    ''' We should reject invocation with both product locator -and- location
        file. One or the other, people!

    '''

    args = ['--product-locator', 'a_locator',
            '--location-file', 'location.json'
            '--output-dir', str(make_tempdir),
            '--profile', TEST_PROFILE]
    with pytest.raises(SystemExit) as exc:
        launch_datafetcher(args, settings.capo_settings)
    assert exc.value.code == ReturnCode.MISSING_SETTING.value['code']

class MockServiceTimeoutReturn:
    ''' Simulates locator request service timeout '''

    @staticmethod
    def run():
        return ReturnCode.LOCATOR_SERVICE_TIMEOUT

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_locator_service_timeout_returns_expected_code(monkeypatch,
                                                       settings,
                                                       make_tempdir):

    def mock_run(*args, **kwargs):
        return MockServiceTimeoutReturn.run()

    args = ['--product-locator', settings.test_data['product_locator'],
            '--output-dir', str(make_tempdir),
            '--profile', TEST_PROFILE]
    monkeypatch.setattr(DataFetcherRefactor, "run", mock_run)
    return_code = launch_datafetcher(args, settings.capo_settings)
    assert return_code == ReturnCode.LOCATOR_SERVICE_TIMEOUT


class MockTooManyServiceRedirectsReturn:

    @staticmethod
    def run():
        return ReturnCode.TOO_MANY_SERVICE_REDIRECTS

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_too_many_service_redirects_returns_expected_code(monkeypatch,
                                                          settings,
                                                          make_tempdir):

    def mock_run(*args, **kwargs):
        return MockTooManyServiceRedirectsReturn().run()

    args = ['--product-locator', settings.test_data['product_locator'],
            '--output-dir', str(make_tempdir),
            '--profile', TEST_PROFILE]
    monkeypatch.setattr(DataFetcherRefactor, "run", mock_run)
    return_code = launch_datafetcher(args, settings.capo_settings)
    assert return_code == ReturnCode.TOO_MANY_SERVICE_REDIRECTS


class MockCatastrophicServiceErrorReturn:

    @staticmethod
    def run():
        return ReturnCode.CATASTROPHIC_REQUEST_ERROR

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_catastrophic_service_error_returns_expected_code(monkeypatch,
                                                          settings,
                                                          make_tempdir):

    def mock_run(*args, **kwargs):
        return MockCatastrophicServiceErrorReturn().run()

    args = ['--product-locator', settings.test_data['product_locator'],
            '--output-dir', str(make_tempdir),
            '--profile', TEST_PROFILE]

    monkeypatch.setattr(DataFetcherRefactor, "run", mock_run)
    return_code = launch_datafetcher(args, settings.capo_settings)
    assert return_code == ReturnCode.CATASTROPHIC_REQUEST_ERROR

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_product_locator_not_found_returns_expected_code(make_tempdir,
                                                         settings):
    args = ['--product-locator', 'not_a_locator',
            '--output-dir', str(make_tempdir),
            '--profile', TEST_PROFILE]
    with pytest.raises(SystemExit) as exc:
        launch_datafetcher(args, settings.capo_settings)
    assert exc.value.code == ReturnCode.PRODUCT_LOCATOR_NOT_FOUND.value

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_unable_to_open_location_file_returns_expected_code(make_tempdir, settings):
    args = ['--location-file', 'location.json',
            '--output-dir', str(make_tempdir),
            '--profile', TEST_PROFILE]
    with pytest.raises(SystemExit) as exc:
        launch_datafetcher(args, settings.capo_settings)
    assert exc.value.code == ReturnCode.CANNOT_OPEN_LOCATION_FILE.value


class MockNgasFetchError:
    ''' Simulates a problem fetching file(s) from NGAS server '''
    @staticmethod
    def run():
        return ReturnCode.NGAS_FETCH_ERROR

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_error_fetching_file_from_ngas_returns_expected_code(monkeypatch,
                                                             settings,
                                                             make_tempdir):

    def mock_run(*args, **kwargs):
        return MockNgasFetchError().run()

    args = ['--product-locator', settings.test_data['product_locator'],
            '--output-dir', str(make_tempdir),
            '--profile', TEST_PROFILE]
    monkeypatch.setattr(DataFetcherRefactor, "run", mock_run)
    return_code = launch_datafetcher(args, settings.capo_settings)
    assert return_code == ReturnCode.NGAS_FETCH_ERROR


class MockSizeMismatchError:
    ''' Simulates a retrieved file of unexpected size '''
    @staticmethod
    def run():
        return ReturnCode.SIZE_MISMATCH

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_unexpected_size_returns_expected_code(monkeypatch,
                                               settings,
                                               make_tempdir):

    def mock_run(*args, **kwargs):
        return MockSizeMismatchError().run()

    args = ['--product-locator', settings.test_data['product_locator'],
            '--output-dir', str(make_tempdir),
            '--profile', TEST_PROFILE]
    monkeypatch.setattr(DataFetcherRefactor, "run", mock_run)
    return_code = launch_datafetcher(args, settings.capo_settings)
    assert return_code == ReturnCode.SIZE_MISMATCH

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_nothing_retrieved_if_dry_locator(make_tempdir, settings):
    ''' Simulates dry run with product locator '''

    args = ['--product-locator', settings.test_data['product_locator'],
            '--output-dir', str(make_tempdir),
            '--profile', TEST_PROFILE,
            '--dry-run'
            ]
    return_code = launch_datafetcher(args, settings.capo_settings)
    assert return_code == 0
    tempdir_files = Path(make_tempdir).iterdir()
    for file in tempdir_files:
        if not str(file).endswith('.log') \
                and not str(file).endswith('.json'):
            pytest.fail('dry run with product locator -DID- fetch files')

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_nothing_retrieved_if_dry_file(make_tempdir, settings):
    ''' dry run with locations file '''

    output_dir = Path(make_tempdir)
    locations_file = get_mini_locations_file(Path(output_dir, _LOCATION_FILENAME))
    args = ['--location-file', str(locations_file),
            '--output-dir', str(make_tempdir),
            '--profile', TEST_PROFILE,
            '--dry-run'
            ]
    launch_datafetcher(args, settings.capo_settings)
    tempdir_files = output_dir.iterdir()
    for file in tempdir_files:
        if not str(file).endswith('.log') \
                and not str(file).endswith('.json'):
            pytest.fail('dry run with locations file -DID- fetch files')

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_force_flag_overwrites_existing_file(make_tempdir, settings):
    top_level = Path(make_tempdir)
    location_file = get_mini_locations_file(top_level / _LOCATION_FILENAME)
    dest_dir = Path(top_level, _EB_EXTERNAL_NAME)
    dest_dir.mkdir(parents=True, exist_ok=True)

    # make a fake file to be overwritten
    fake_file = dest_dir / _ASDM_XML
    with open(fake_file, 'w') as to_write:
        to_write.write('alas, my days are numbered')

    args = ['--location-file', str(location_file),
            '--profile', TEST_PROFILE,
            '--output-dir', str(top_level),
            '--force']
    try:
        launch_datafetcher(args, settings.capo_settings)
    except SystemExit as ex:
        pytest.fail(f'{ex}')
    except Exception as exc:
        pytest.fail(f'{exc}')

    sizes = dict()

    # go thru destination directory recursively
    # and get everybody's size
    for file in dest_dir.rglob('*'):
        sizes[str(file)] = file.stat().st_size
    assert len(sizes) == 37
    fake_size = fake_file.stat().st_size
    assert fake_size == 9339

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_no_overwrite_without_force(make_tempdir, settings):
    top_level = Path(make_tempdir)
    location_file = get_mini_locations_file(top_level / _LOCATION_FILENAME)
    dest_dir = Path(top_level, _EB_EXTERNAL_NAME)
    dest_dir.mkdir(parents=True, exist_ok=True)

    # make a fake file to be overwritten
    fake_file = dest_dir / _ASDM_XML
    with open(fake_file, 'w') as to_write:
        to_write.write('alas, my days are numbered')

    args = ['--location-file', str(location_file),
            '--profile', TEST_PROFILE,
            '--output-dir', str(top_level)
            ]

    with pytest.raises(SystemExit) as exc:
        launch_datafetcher(args, settings.capo_settings)
    assert exc.value.code == ReturnCode.NGAS_FETCH_ERROR.value['code']

    sizes = dict()
    for file in dest_dir.rglob('*'):
        sizes[str(file)] = file.stat().st_size
    assert len(sizes) < 37
    fake_size = fake_file.stat().st_size
    assert fake_size == 26


@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_more_output_when_verbose(make_tempdir, settings):
    top_level = Path(make_tempdir)
    location_file = get_mini_locations_file(top_level / _LOCATION_FILENAME)

    args = ['--location-file', str(location_file),
            '--profile', TEST_PROFILE,
            '--output-dir', str(make_tempdir),
            '--verbose']

    return_code = launch_datafetcher(args, settings.capo_settings)
    assert return_code == ReturnCode.SUCCESS.value['code']

    num_files_expected = 37
    retrieved = [file for file in top_level.rglob('*')
                 if file.is_file()]
    assert num_files_expected == len(retrieved) - 2

    verbose_logfile = None
    for file in retrieved:
        if str(file).endswith('.log'):
            verbose_logfile = file
            break

    assert verbose_logfile is not None
    verbose_log_size = verbose_logfile.stat().st_size
    assert verbose_log_size > 0

    # get rid of all the files we downloaded, plus the log
    deleted = [file.unlink() for file in retrieved
               if not str(file).endswith('.json')]
    assert len(deleted) >= num_files_expected

    ### same download, but without verbose logging
    args = ['--location-file', str(location_file),
            '--profile', TEST_PROFILE,
            '--output-dir', str(make_tempdir)]

    return_code = launch_datafetcher(args, settings.capo_settings)
    assert return_code == ReturnCode.SUCCESS.value['code']

    retrieved = [file for file in top_level.rglob('*')]
    assert len(retrieved) == num_files_expected + 3

    logfile = None
    for file in retrieved:
        if str(file).endswith('.log'):
            logfile = file
            break
    assert logfile is not None

    logsize = logfile.stat().st_size
    # successful download, non-verbose logging,
    # should result in zero-size log file
    assert logsize == 0

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_copy_attempt_throws_sys_exit_service_error(make_tempdir, settings):
    local_exec_site = settings.capo_settings['execution_site']

    # use site from non-local profile to guarantee copy attempt
    settings.capo_settings['execution_site'] = ExecutionSite.DSOC

    args = ['--product-locator', settings.test_data['product_locator'],
            '--output-dir', str(make_tempdir),
            '--profile', 'nmprod']
    namespace = get_arg_parser().parse_args(args)
    fetcher = DataFetcherRefactor(namespace, settings.capo_settings)

    servers_report = fetcher.servers_report
    for server in servers_report:
        entry = servers_report[server]
        assert entry['retrieve_method'].value == RetrievalMode.COPY.value

    # let's try just one file so we're not sitting here all day
    for server in servers_report:
        entry = servers_report[server]
        servers_report = {server: entry}
        fetcher.servers_report = servers_report
        assert fetcher.servers_report[server] is not None
        break
    files = fetcher.servers_report[server]['files']
    fetcher.servers_report[server]['files'] = [files[0]]

    try:
        with pytest.raises(SystemExit) as exc:
            fetcher.run()
        assert exc.value.code == \
               ReturnCode.CATASTROPHIC_REQUEST_ERROR.value['code']
    finally:
        settings.capo_settings['execution_site'] = local_exec_site

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_dies_with_bad_server_info(make_tempdir, settings):
    location_file = get_locations_file('VLA_BAD_SERVER')
    args = ['--profile', TEST_PROFILE, '--output-dir', str(make_tempdir),
            '--location-file', str(location_file)]
    with pytest.raises(SystemExit) as exc:
        launch_datafetcher(args, settings.capo_settings)
    error = ReturnCode.NGAS_FETCH_ERROR
    expected = error.value['code']
    assert exc.value.code == expected

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_missing_setting_error_on_bad_destination(settings):
    args = ['--profile', TEST_PROFILE,
            '--product-locator', settings.test_data['product_locator'],
            '--output-dir', 'floob']
    with pytest.raises(SystemExit) as exc:
        launch_datafetcher(args, settings.capo_settings)
    assert exc.value.code == ReturnCode.MISSING_SETTING.value


def write_fake_file(destination: Path, file_info: dict):
    filename = file_info['ngas_file_id']
    path = Path(destination, filename)
    with open(path, 'w') as file:
        file.write(f'{str(file_info["size"])}\n')


class MockSuccessfulFetchReturn:

    @staticmethod
    def run():
        return 0

@pytest.fixture
def mock_successful_fetch_run(monkeypatch):

    def mock_run(*args, **kwargs):
        return MockSuccessfulFetchReturn().run()

    monkeypatch.setattr(DataFetcherRefactor, "run", mock_run)

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_gets_vlbas_from_report_file(mock_successful_fetch_run,
                                     make_tempdir,
                                     settings):

    location_file = get_locations_file('VLBA_EB')
    args = ['--profile', TEST_PROFILE, '--output-dir', str(make_tempdir),
            '--location-file', str(location_file)]
    fetcher = DataFetcherRefactor(get_arg_parser().parse_args(args),
                                  settings.capo_settings)
    servers_report = fetcher.servers_report
    assert len(servers_report) == 1

    return_code = fetcher.run()
    assert return_code == 0

    dest_dir = Path(make_tempdir)
    file_info_dict = dict()
    for server in servers_report.items():
        assert server[0] == 'nmngas03.aoc.nrao.edu:7777'
        values = server[1]
        assert values['location'] == Location.DSOC.value
        assert values['cluster'] == Cluster.DSOC.value
        assert values['retrieve_method'] == RetrievalMode.STREAM
        file_values = values['files']
        assert len(file_values) == 16
        for filename in file_values:
            write_fake_file(dest_dir, filename)
            file_info_dict[filename['ngas_file_id']] = filename

    datafetcher = DataFetcherRefactor(get_arg_parser().parse_args(args),
                                      capo_settings)
    return_code = datafetcher.run()
    assert return_code == 0

    for filename in file_info_dict:
        path = Path(dest_dir, filename)
        assert path.is_file()
        contents = path.read_text().strip()
        assert int(contents) == file_info_dict[filename]['size']

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_gets_large_vla_ebs_from_report_file(mock_successful_fetch_run,
                                             make_tempdir,
                                             settings):
    location_file = get_locations_file('VLA_SMALL_EB')
    args = ['--profile', TEST_PROFILE, '--output-dir', str(make_tempdir),
            '--location-file', str(location_file)]
    fetcher = DataFetcherRefactor(get_arg_parser().parse_args(args),
                                  settings.capo_settings)
    servers_report = fetcher.servers_report
    assert len(servers_report) == 2

    return_code = fetcher.run()
    assert return_code == 0

    server_file_count = {'nmngas03.aoc.nrao.edu:7777': 0,
                         'nmngas04.aoc.nrao.edu:7777': 0}
    dest_dir = Path(make_tempdir)
    file_list = list()
    for server in servers_report.items():
        server_url = server[0]
        assert server_url in server_file_count.keys()
        values = server[1]
        assert values['location'] == Location.DSOC.value
        assert values['cluster'] == Cluster.DSOC.value
        assert values['retrieve_method'] == RetrievalMode.STREAM
        file_values = values['files']
        server_file_count[server_url] += len(file_values)

        for filename in file_values:
            write_fake_file(dest_dir, filename)
            file_list.append(values)

    assert server_file_count['nmngas03.aoc.nrao.edu:7777'] == 3
    assert server_file_count['nmngas04.aoc.nrao.edu:7777'] == 41

    datafetcher = DataFetcherRefactor(get_arg_parser().parse_args(args),
                                      capo_settings)
    return_code = datafetcher.run()
    assert return_code == 0

    found_count = 0
    for file_info in file_list:
        for file in file_info['files']:
            filename = file['ngas_file_id']
            path = Path(dest_dir, filename)
            assert path.is_file()
            contents = path.read_text().strip()
            assert int(contents) == file['size']
        found_count += 1

    assert found_count == len(file_list)

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_gets_images_from_report_file(mock_successful_fetch_run,
                                      make_tempdir,
                                      settings):
    location_file = get_locations_file('IMG')
    args = ['--profile', TEST_PROFILE, '--output-dir', str(make_tempdir),
            '--location-file', str(location_file)]
    fetcher = DataFetcherRefactor(get_arg_parser().parse_args(args),
                                  settings.capo_settings)
    servers_report = fetcher.servers_report
    assert len(servers_report) == 2

    server_file_count = {'nmngas01.aoc.nrao.edu:7777': 0,
                         'nmngas02.aoc.nrao.edu:7777': 0}
    dest_dir = Path(make_tempdir)
    file_list = list()
    for server in servers_report.items():
        server_url = server[0]
        assert server_url in server_file_count.keys()
        values = server[1]
        assert values['location'] == Location.DSOC.value
        assert values['cluster'] == Cluster.DSOC.value
        assert values['retrieve_method'] == RetrievalMode.STREAM
        file_values = values['files']
        server_file_count[server_url] += len(file_values)

        for filename in file_values:
            write_fake_file(dest_dir, filename)
            file_list.append(values)

    for server_url, count in server_file_count.items():
        assert count == 1

    return_code = fetcher.run()
    assert return_code == 0

    found_count = 0
    for file_info in file_list:
        for file in file_info['files']:
            filename = file['ngas_file_id']
            path = Path(dest_dir, filename)
            assert path.is_file()
            contents = path.read_text().strip()
            assert int(contents) == file['size']
        found_count += 1

    assert found_count == len(file_list)

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_gets_calibration_from_report_file(mock_successful_fetch_run,
                                           make_tempdir,
                                           settings):
    location_file = get_locations_file('CALIBRATION')
    args = ['--profile', TEST_PROFILE, '--output-dir', str(make_tempdir),
            '--location-file', str(location_file)]
    fetcher = DataFetcherRefactor(get_arg_parser().parse_args(args),
                                  settings.capo_settings)
    servers_report = fetcher.servers_report
    assert len(servers_report) == 1

    fake_file = None
    file_info = None
    # (there will be just one file, therefore iteration)
    for server in servers_report.items():
        metadata = server[1]
        destination = Path(make_tempdir)
        file_info = metadata['files'][0]
        fake_file = Path(destination, file_info['ngas_file_id'])
        write_fake_file(destination, file_info)

    assert fake_file.is_file()
    contents = fake_file.read_text().strip()
    assert int(contents) == file_info['size']

@pytest.mark.skipif(not RUN_ALL, reason='debug')
def test_gets_calibration_from_locator(mock_successful_fetch_run,
                                       make_tempdir,
                                       settings):
    external_name = LOCATION_REPORTS['CALIBRATION']['external_name']
    product_locator = ProductLocatorLookup(
        settings.db_settings).look_up_locator_for_ext_name(external_name)
    args = ['--product-locator', product_locator,
            '--output-dir', str(make_tempdir), '--profile', TEST_PROFILE]
    namespace = get_arg_parser().parse_args(args)
    fetch = DataFetcherRefactor(namespace, settings.capo_settings)
    report_files = fetch.locations_report.files_report['files']
    assert len(report_files) == 1

    file_spec = report_files[0]
    relative_path = file_spec['relative_path']
    assert external_name == relative_path
    assert relative_path == file_spec['subdirectory']
    destination = Path(make_tempdir)/relative_path
    destination.mkdir()
    write_fake_file(destination, file_spec)

    fake_file = Path(destination, file_spec['ngas_file_id'])
    assert fake_file.is_file()
    contents = fake_file.read_text().strip()
    assert int(contents) == file_spec['size']


class Settings:

    def __init__(self, capo_settings, db_settings, test_data):
        self.capo_settings = capo_settings
        self.db_settings = db_settings
        self.test_data = test_data
